#!/bin/sh
#

# ==============================================================================
# Init
# ==============================================================================
JVM_BIN=jamvm
#JVM_BIN=cacao

JVM_OPT=
#JVM_OPT="-i -ifconv"

JVM="/usr/bin/${JVM_BIN} ${JVM_OPT} -Djava.specification.version=1.4"

MEM="-Xms16M -Xmx48M -Xss64K"
if [ -x /usr/bin/awk ]
then
    MEMT=`/usr/bin/awk '$1 ~ /MemTotal/ { print $2 }' /proc/meminfo`
    if [ ${MEMT} -ge 460000 ] ; then
        # echo "512MB or more system"
        MEM="-Xms64M -Xmx256M -Xss64K"
    elif [ ${MEMT} -ge 200000 ] ; then
        # echo "256MB-512MB system"
        MEM="-Xms48M -Xmx128M -Xss64K"
    else
        # echo "Less than 256MB system"
        MEM="-Xms16M -Xmx48M -Xss64K"
    fi
fi

JAVA="${JVM} ${MEM} -Dlog4j.configuration=file:/tpinstaller/log4j.properties"
JAVA_DBG="${JVM} ${MEM} -Dlog4j.configuration=file:/tpinstaller/log4j-dbg.properties"

DIR_STATE=/var/run
DIR_LOG=/var/log
DIR_TPINST=/tpinstaller
DIR_LIB=$DIR_TPINST/libs
DIR_WORK=$DIR_TPINST/work

if [ -f /etc/tp-device.conf-default ] ; then
    . /etc/tp-device.conf-default
fi
if [ -f /etc/tp-device.conf ] ; then
    . /etc/tp-device.conf
fi
if [ -f /etc/tp-product.conf-default ] ; then
    . /etc/tp-product.conf-default
fi
if [ -f /etc/tp-product.conf ] ; then
    . /etc/tp-product.conf
fi

VERSION_EIBBUS="BUS:${FIRMWARE_BUS}"
VERSION_EIB="EIB:${FIRMWARE_EIB}"
VERSION_TPI="TP:${FIRMWARE_TP}"
SERIAL="${PRODUCT_SERIAL}"

VERSION="${VERSION_EIBBUS};${VERSION_EIB};${VERSION_TPI}"

if [ ! -d $DIR_WORK ]
then
    # create WORK directory if not exist!
    mkdir -p $DIR_WORK
fi

JAR_LOG4J=$DIR_LIB/log4j-1.2.8.jar
JAR_USB4J=$DIR_LIB/libusb4j.jar
JAR_EIBBUS=$DIR_LIB/eibbus.jar
JAR_EIB=$DIR_LIB/eib.jar
JAR_TTASPIRIN=$DIR_LIB/ttaspirin.jar
JAR_XSTREAM=$DIR_LIB/xstream-1.4.2.jar
JAR_XPP=$DIR_LIB/xpp3_min-1.1.4c.jar
JAR_TPINST=$DIR_LIB/tpinstaller.jar
JAR_MAIL=$DIR_LIB/mail.jar
JAR_MAILC=$DIR_LIB/commons-email-1.2.jar
JAR_DNSJAVA=$DIR_LIB/dnsjava-2.1.3.jar

RUN_CP=$JAR_LOG4J:$JAR_USB4J:$JAR_EIBBUS:$JAR_EIB:$JAR_XSTREAM:$JAR_XPP:$JAR_TPINST:$JAR_MAIL:$JAR_MAILC:$JAR_DNSJAVA:$JAR_TTASPIRIN
BOOT_CP="-Xbootclasspath/p:$DIR_LIB/activation.jar"
MAIN_CLASS=com.ttechgroup.cdinnovation.tp.objectcanvas.standalone.MainFrame
PRJRUN_CLASS=com.ttechgroup.cdinnovation.tp.objectcanvas.standalone.awt.ObjectCanvasFrameAWT
SPLASH_CLASS=com.ttechgroup.splash.Starter
OUT_FILE=$DIR_LOG/tplog.out

RUN_PROJECT=$2

# MIDI Out device
export MIDI_PORT='TiMidity port 0'

# ==============================================================================
# Commands
# ==============================================================================

ss_reset() {
    if [ -x /usr/bin/xset ]
    then
        export DISPLAY=":0"
        /usr/bin/xset s reset
    fi
}

tp_stop() {
    echo "stop" > $DIR_STATE/jamvm_state;
    echo "" > $DIR_STATE/eib_state;

    /bin/pidof ${JVM_BIN} 1>/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        # there are running JVM!
        echo "app-stop" | /usr/bin/nc -w 10 -n 127.0.0.1 7000 1>/dev/null 2>&1
        # wait max 10 seconds more for JVM to stop gracefully
        for cnt in 1 2 3 4 5 6 7 8 9 10 ; do
            /bin/pidof ${JVM_BIN} 1>/dev/null 2>&1
            if [ $? -ne 0 ]
            then
                # JVM is stopped gracefully!
                break
            fi
            /bin/sleep 1
        done
        /bin/pidof ${JVM_BIN} 1>/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            # JVM is running! send TERM signal!
            # also send QUIT signall to knowh wich threads are not finished yet!
            /usr/bin/killall -QUIT ${JVM_BIN} 1>/dev/null 2>&1
            /usr/bin/killall -INT ${JVM_BIN} 1>/dev/null 2>&1
            /bin/sleep 3
            /usr/bin/killall -KILL ${JVM_BIN} 1>/dev/null 2>&1
        fi
    fi
    # check for linphonec & ttplay
    /bin/pidof linphonec ttplay 1>/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        # there are running linphonec and/or ttplay!
        /usr/bin/killall -INT linphonec 1>/dev/null 2>&1
        /usr/bin/killall -INT ttplay 1>/dev/null 2>&1
        /bin/sleep 3
        /usr/bin/killall -KILL linphonec 1>/dev/null 2>&1
        /usr/bin/killall -KILL ttplay 1>/dev/null 2>&1
    fi
    /bin/rm -rf /tmp/Project_1
}

tp_start() {
    tp_stop

    if [ "${RUN_PROJECT}x" = "x" ]
    then
        if [ -f $DIR_WORK/current_project ]
        then
            RUN_PROJECT=$(/bin/cat $DIR_WORK/current_project)
        fi
        if [ "${RUN_PROJECT}x" = "x" ]
        then
            echo "Project is not defined! Start failed!"
            exit 1
        fi
    fi

    if [ -f $RUN_PROJECT ] 
    then
        echo -n "run" > $DIR_STATE/jamvm_state;
        echo -n "" > $DIR_STATE/eib_state;
        echo -n $RUN_PROJECT > $DIR_WORK/current_project

        ss_reset

        /bin/rm -rf /tmp/Project_1
        export DISPLAY=":0"
        $JAVA $BOOT_CP -cp $RUN_CP -Dapp=$MAIN_CLASS $SPLASH_CLASS $RUN_PROJECT > $OUT_FILE 2>&1 &
    else
        rm -f $DIR_WORK/current_project
        echo "Project is not defined! Start failed!"
        exit 1
    fi
}

tp_debug() {
    tp_stop

    if [ "${RUN_PROJECT}x" = "x" ]
    then
        if [ -f $DIR_WORK/current_project ]
        then
            RUN_PROJECT=$(/bin/cat $DIR_WORK/current_project)
        fi
        if [ "${RUN_PROJECT}x" = "x" ]
        then
            echo "Project is not defined! Start failed!"
            exit 1
        fi
    fi

    if [ -f $RUN_PROJECT ] 
    then
        echo -n "run" > $DIR_STATE/jamvm_state;
        echo -n "" > $DIR_STATE/eib_state;
        echo -n $RUN_PROJECT > $DIR_WORK/current_project

        ss_reset

        /bin/rm -rf /tmp/Project_1
        export DISPLAY=":0"
        $JAVA_DBG $BOOT_CP -cp $RUN_CP -Dapp=$MAIN_CLASS $SPLASH_CLASS $RUN_PROJECT 
    else
        rm -f $DIR_WORK/current_project
        echo "Project is not defined! Start failed!"
        exit 1
    fi
}

tp_run() {
    tp_stop

    if [ "${RUN_PROJECT}x" = "x" ]
    then
        if [ -f $DIR_WORK/current_project ]
        then
            RUN_PROJECT=$(/bin/cat $DIR_WORK/current_project)
        fi
        if [ "${RUN_PROJECT}x" = "x" ]
        then
            echo "Project is not defined! Start failed!"
            exit 1
        fi
    fi

    if [ -f $RUN_PROJECT ] 
    then
        echo -n "run" > $DIR_STATE/jamvm_state;
        echo -n "" > $DIR_STATE/eib_state;
        echo -n $RUN_PROJECT > $DIR_WORK/current_project

        ss_reset

        /bin/rm -rf /tmp/Project_1
        export DISPLAY=":0"
        $JAVA $BOOT_CP -cp $RUN_CP -Dapp=$PRJRUN_CLASS $SPLASH_CLASS $RUN_PROJECT > $OUT_FILE 2>&1 &
    else
        rm -f $DIR_WORK/current_project
        echo "Project is not defined! Start failed!"
        exit 1
    fi
}

tp_status() {
    if [ -f $DIR_STATE/jamvm_state ] 
    then
        cat $DIR_STATE/jamvm_state
    else
        echo "stop"
    fi
}

tp_restart() {
    tp_start
}

tp_genuid() {
    $JAVA -cp $RUN_CP com.ttechgroup.embed.gui.GenerateDeviceConf
}

# ==============================================================================
# Main
# ==============================================================================

cd $DIR_TPINST
case "$1" in
    start)
        # tp_start
        nohup $DIR_TPINST/tpjvmd.sh start > /tmp/nohup.out &
        ;;
    debug)
        tp_debug
        ;;
    run)
        # tp_run
        nohup $DIR_TPINST/tpjvmd.sh run > /tmp/nohup.out &
        ;;
    stop)
        tp_stop
        ;;
    status)
        tp_status
        ;;
    restart|reload)
        tp_restart
        ;;
    genuid)
        tp_genuid
        ;;
    version)
        echo ${VERSION}
        ;;
    serial)
        echo ${SERIAL}
        ;;
    *)
        echo $"Usage: $0 {start|run|stop|status|restart|version} [project]"
        echo $"  start   - starts the project in setup page"
        echo $"  run     - runs the project in main page"
        echo $"  debug   - starts the projet in setup page, debug mode to stdout"
        echo $"  stop    - stops the project"
        echo $"  status  - print the status of the project. stop or run"
        echo $"  restart - restarts the project"
        echo $"  genuid  - generate /etc/tp-device.conf file"
        echo $"  version - print the version of the firmware in format <EIBBUS>:<EIB>:<TPINST>"
        echo $"  serial  - print the serial number of the product"
        exit 1
esac

exit $?

