#!/bin/sh
#

# ==============================================================================
# Init
# ==============================================================================
JVM_BIN=jamvm
#JVM_BIN=cacao

JVM_OPT="-Xcodemem:unlimited"
#JVM_OPT="-i -ifconv"

JVM="/usr/bin/${JVM_BIN} ${JVM_OPT} -Djava.specification.version=1.4"

MEM="-Xms16M -Xmx48M -Xss64K"
if [ -x /usr/bin/awk ]
then
    MEMT=`/usr/bin/awk '$1 ~ /MemTotal/ { print $2 }' /proc/meminfo`
    if [ ${MEMT} -ge 460000 ] ; then
        # echo "512MB or more system"
        MEM="-Xms64M -Xmx256M -Xss64K"
    elif [ ${MEMT} -ge 200000 ] ; then
        # echo "256MB-512MB system"
        MEM="-Xms48M -Xmx128M -Xss64K"
    else
        # echo "Less than 256MB system"
        MEM="-Xms16M -Xmx48M -Xss64K"
    fi
fi

JAVA="${JVM} ${MEM} -Dlog4j.configuration=file:/tpinstaller/log4j.properties"
JAVA_DBG="${JVM} ${MEM} -Dlog4j.configuration=file:/tpinstaller/log4j-dbg.properties"

DIR_STATE=/var/run
DIR_LOG=/var/log
DIR_TPINST=/tpinstaller
DIR_LIB=$DIR_TPINST/libs
DIR_WORK=$DIR_TPINST/work

if [ -f /etc/tp-device.conf-default ] ; then
    . /etc/tp-device.conf-default
fi
if [ -f /etc/tp-device.conf ] ; then
    . /etc/tp-device.conf
fi
if [ -f /etc/tp-product.conf-default ] ; then
    . /etc/tp-product.conf-default
fi
if [ -f /etc/tp-product.conf ] ; then
    . /etc/tp-product.conf
fi

VERSION_EIBBUS="BUS:${FIRMWARE_BUS}"
VERSION_EIB="EIB:${FIRMWARE_EIB}"
VERSION_TPI="TP:${FIRMWARE_TP}"
SERIAL="${PRODUCT_SERIAL}"

VERSION="${VERSION_EIBBUS};${VERSION_EIB};${VERSION_TPI}"

if [ ! -d $DIR_WORK ]
then
    # create WORK directory if not exist!
    mkdir -p $DIR_WORK
fi

JAR_LOG4J=$DIR_LIB/log4j-1.2.8.jar
JAR_USB4J=$DIR_LIB/libusb4j.jar
JAR_EIBBUS=$DIR_LIB/eibbus.jar
JAR_EIB=$DIR_LIB/eib.jar
JAR_TTASPIRIN=$DIR_LIB/ttaspirin.jar
JAR_XSTREAM=$DIR_LIB/xstream-1.4.2.jar
JAR_XPP=$DIR_LIB/xpp3_min-1.1.4c.jar
JAR_TPINST=$DIR_LIB/tpinstaller.jar
JAR_MAIL=$DIR_LIB/mail.jar
JAR_MAILC=$DIR_LIB/commons-email-1.2.jar
JAR_DNSJAVA=$DIR_LIB/dnsjava-2.1.3.jar

RUN_CP=$JAR_LOG4J:$JAR_USB4J:$JAR_EIBBUS:$JAR_EIB:$JAR_XSTREAM:$JAR_XPP:$JAR_TPINST:$JAR_MAIL:$JAR_MAILC:$JAR_DNSJAVA:$JAR_TTASPIRIN
BOOT_CP="-Xbootclasspath/p:$DIR_LIB/activation.jar"
MAIN_CLASS=com.ttechgroup.cdinnovation.tp.objectcanvas.standalone.MainFrame
PRJRUN_CLASS=com.ttechgroup.cdinnovation.tp.objectcanvas.standalone.awt.ObjectCanvasFrameAWT
SPLASH_CLASS=com.ttechgroup.splash.Starter
OUT_FILENAME=tplog.out
OUT_FILE=$DIR_LOG/$OUT_FILENAME

RUN_PROJECT=$2

if [ "${RUN_PROJECT}x" = "x" ]
then
    if [ -f $DIR_WORK/current_project ]
    then
        RUN_PROJECT=$(/bin/cat $DIR_WORK/current_project)
    fi
    if [ "${RUN_PROJECT}x" = "x" ]
    then
        echo "Project is not defined! Start failed!"
        exit 1
    fi
fi

if [ ! -f $RUN_PROJECT ]
then
    rm -f $DIR_WORK/current_project
    echo "Project file does not exist! Start failed!"
    exit 1
fi

# MIDI Out device
export MIDI_PORT='TiMidity port 0'
# X Server display
export DISPLAY=":0"

# ==============================================================================
# Commands
# ==============================================================================

tp_logrot() {
    rm -f $OUT_FILE.5
    INDEX=5
    while [ $INDEX -gt 1 ] ; do
        let NEXT_INDEX=INDEX-1
        if [ -f $OUT_FILE.$NEXT_INDEX ] ; then 
            mv $OUT_FILE.$NEXT_INDEX $OUT_FILE.$INDEX
        fi
        let INDEX=NEXT_INDEX
    done
    if [ -f $OUT_FILE ] ; then
        mv $OUT_FILE $OUT_FILE.1
    fi
}

tp_start() {
    echo -n "run" > $DIR_STATE/jamvm_state;
    echo -n "" > $DIR_STATE/eib_state;
    echo -n $RUN_PROJECT > $DIR_WORK/current_project

    /bin/rm -rf /tmp/Project_1

    tp_logrot
    echo "=======================================================================" > $OUT_FILE
    echo -n "Start project on " >> $OUT_FILE
    echo `date` >> $OUT_FILE

    while true ; do
        #ulimit -s 16384
        $JAVA $BOOT_CP -cp $RUN_CP -Dapp=$MAIN_CLASS $SPLASH_CLASS $RUN_PROJECT >> $OUT_FILE 2>&1
        JAVA_EXIT_CODE=$?
        echo "=======================================================================" >> $OUT_FILE

        # check for linphonec & ttplay
        /bin/pidof linphonec ttplay 1>/dev/null 2>&1
        if [ $? -eq 0 ] ; then
            # there are running linphonec and/or ttplay!
            /usr/bin/killall -INT linphonec 1>/dev/null 2>&1
            /usr/bin/killall -INT ttplay 1>/dev/null 2>&1
            /bin/sleep 3
            /usr/bin/killall -KILL linphonec 1>/dev/null 2>&1
            /usr/bin/killall -KILL ttplay 1>/dev/null 2>&1
        fi

        if [ $JAVA_EXIT_CODE -lt 128 ] ; then
            // normal exit!
            echo -n "Project stops successfuly with exit code " >> $OUT_FILE
            echo -n $JAVA_EXIT_CODE >> $OUT_FILE
            echo -n " on " >> $OUT_FILE
            echo `date` >> $OUT_FILE
            break
        fi
 
	// exited by signal!
	let JAVA_EXIT_SIGNAL=JAVA_EXIT_CODE-128
	echo -n "Project stops by receiving an signal " >> $OUT_FILE
        echo -n $JAVA_EXIT_SIGNAL >> $OUT_FILE
        echo -n " on " >> $OUT_FILE
        echo `date` >> $OUT_FILE

	SIGNAL_NAME="UNKNOWN"
        RESTART=false
	case $JAVA_EXIT_SIGNAL in
            4)
                SIGNAL_NAME="SIGILL"
                RESTART=true
                ;;
            8)
                SIGNAL_NAME="SIGFPE"
                RESTART=true
                ;;
            11)
                SIGNAL_NAME="SIGSEGV"
                RESTART=true
                ;;
            16)
                SIGNAL_NAME="SIGSTKFLT"
                RESTART=true
                ;;
        esac
        if [ ! $RESTART ] ; then
            echo "This signal is not market as NEED_RESTART signal! Stop project!" >> $OUT_FILE
            break
        fi
        echo "Restarting project! Signal is" $SIGNAL_NAME  >> $OUT_FILE
        echo "=======================================================================" >> $OUT_FILE
        /bin/rm -rf /tmp/Project_1
	# break
    done
}

tp_run() {
    echo -n "run" > $DIR_STATE/jamvm_state;
    echo -n "" > $DIR_STATE/eib_state;
    echo -n $RUN_PROJECT > $DIR_WORK/current_project

    /bin/rm -rf /tmp/Project_1

    tp_logrot
    echo "=======================================================================" > $OUT_FILE
    echo -n "Start project on " >> $OUT_FILE
    echo `date` >> $OUT_FILE

    while true ; do
        #ulimit -s 16384
        $JAVA $BOOT_CP -cp $RUN_CP -Dapp=$PRJRUN_CLASS $SPLASH_CLASS $RUN_PROJECT >> $OUT_FILE 2>&1
        JAVA_EXIT_CODE=$?
        echo "=======================================================================" >> $OUT_FILE

        # check for linphonec & ttplay
        /bin/pidof linphonec ttplay 1>/dev/null 2>&1
        if [ $? -eq 0 ] ; then
            # there are running linphonec and/or ttplay!
            /usr/bin/killall -INT linphonec 1>/dev/null 2>&1
            /usr/bin/killall -INT ttplay 1>/dev/null 2>&1
            /bin/sleep 3
            /usr/bin/killall -KILL linphonec 1>/dev/null 2>&1
            /usr/bin/killall -KILL ttplay 1>/dev/null 2>&1
        fi

        if [ $JAVA_EXIT_CODE -lt 128 ] ; then
            // normal exit!
            echo -n "Project stops successfuly with exit code " >> $OUT_FILE
            echo -n $JAVA_EXIT_CODE >> $OUT_FILE
            echo -n " on " >> $OUT_FILE
            echo `date` >> $OUT_FILE
            break
        fi
 
	// exited by signal!
	let JAVA_EXIT_SIGNAL=JAVA_EXIT_CODE-128
	echo -n "Project stops by receiving an signal " >> $OUT_FILE
        echo -n $JAVA_EXIT_SIGNAL >> $OUT_FILE
        echo -n " on " >> $OUT_FILE
        echo `date` >> $OUT_FILE

	SIGNAL_NAME="UNKNOWN"
        RESTART=false
	case $JAVA_EXIT_SIGNAL in
            4)
                SIGNAL_NAME="SIGILL"
                RESTART=true
                ;;
            8)
                SIGNAL_NAME="SIGFPE"
                RESTART=true
                ;;
            11)
                SIGNAL_NAME="SIGSEGV"
                RESTART=true
                ;;
            16)
                SIGNAL_NAME="SIGSTKFLT"
                RESTART=true
                ;;
        esac
        if [ ! $RESTART ] ; then
            echo "This signal is not market as NEED_RESTART signal! Stop project!" >> $OUT_FILE
            break
        fi
        echo "Restarting project! Signal is" $SIGNAL_NAME  >> $OUT_FILE
        echo "=======================================================================" >> $OUT_FILE
        /bin/rm -rf /tmp/Project_1
	# break
    done
}

# ==============================================================================
# Main
# ==============================================================================

cd $DIR_TPINST
case "$1" in
    start)
        tp_start
        ;;
    run)
        tp_run
        ;;
    *)
        echo "Usage: $0 {start|run} [project]"
        echo "  start   - starts the project in setup page"
        echo "  run     - runs the project in main page"
        exit 1
	;;
esac

exit $?
