#!/bin/sh
#

watchdog() {
    # Get the configuration information
    if [ -f /etc/tp-locale.conf-default ] ; then                                     
        . /etc/tp-locale.conf-default
    fi
    if [ -f /etc/tp-locale.conf ] ; then
        . /etc/tp-locale.conf
    fi
    if [ -f /etc/tp-wdog.conf-default ] ; then
        . /etc/tp-wdog.conf-default
    fi
    if [ -f /etc/tp-wdog.conf ] ; then
        . /etc/tp-wdog.conf
    fi

    REBOOT=

    if [ "x${WDOG_REBOOT_UPTIME}" != "x0" ] ; then
        UPTIME_S=`cat /proc/uptime | awk '{print int($1)}'`
        UPTIME_H=`echo ${UPTIME_S} | awk '{print int($1/3600)}'`
        UPTIME_D=`echo ${UPTIME_H} | awk '{print int($1/24)}'`
        if [ "${UPTIME_D}" -ge "${WDOG_REBOOT_UPTIME}" ] ; then
            # we reach uptime for reboot! check the right time for this!
            #echo "Uptime for reboot reched!"
            TIME_UTC_DS=`date +"%H %M %S" | awk '{print int($1*3600+$2*60+$3)}'`
            TIME_DS=`echo ${TIME_UTC_DS} ${LOCALE_GMTOFFSET} | awk '{s=int($1+$2); if (s<0) s=s+86400; if (s>=86400) s=s-86400; print s}'`
            TIME_H=`echo ${TIME_DS} | awk '{print int($1/3600)}'`
            TIME_M=`echo ${TIME_DS} | awk '{print int($1/60)%60}'`
            TIME_S=`echo ${TIME_DS} | awk '{print $1%60}'`
            if [ "${TIME_H}" -eq "${WDOG_REBOOT_TIME}" ] && [ "${TIME_M}" -le "10" ] ; then
                REBOOT="Uptime reached ${UPTIME_D} days!"
            fi
        fi 
    fi
    if [ "x${WDOG_REBOOT_LOWMEM}" != "x0" ] ; then
        MEM_TOTAL=`awk '$1 ~ /MemTotal/ { print $2 }' /proc/meminfo`
        MEM_FREE=`awk '$1 ~ /MemFree/ { print $2 }' /proc/meminfo`
        MEM_USED_P=`echo ${MEM_TOTAL} ${MEM_FREE} | awk '{print int((100*($1-$2))/$1);}'`
        #echo "Memory Used: ${MEM_USED_P}%"
        if [ "${MEM_USED_P}" -ge "${WDOG_REBOOT_LOWMEM}" ] ; then
            REBOOT="Low-memory barrier of ${WDOG_REBOOT_LOWMEM}% is reached! Current used memory is ${MEM_USED_P}%" 
        fi
    fi
    if [ "x${WDOG_REBOOT_EMI}" != "x0" ] ; then
        dmesg | grep "EMI\\?" > /dev/null
        RES=$?
        if [ "${RES}" -eq "0" ] ; then
            REBOOT="Electical Magnetic Interference is detected on USB Port!"
        fi
    fi
    if [ "x${WDOG_REBOOT_JVM}" != "x0" ] ; then
        dmesg | grep "jamvm\\[" > /dev/null
        RES=$?
        if [ "${RES}" -eq "0" ] ; then
            grep "daemon\\.err\|user\\.err" /tmp/messages > /tmp/ttwdog.reason
            REBOOT="JVM Critical Crash detected!"
        fi
    fi

    if [ -n "${REBOOT}" ] ; then
        echo "Reboot reason: ${REBOOT}"
        # logrotate of previous watchdog reboots
        rm -f /tpinstaller/work/watchdog.log.5
        mv /tpinstaller/work/watchdog.log.4 /tpinstaller/work/watchdog.log.5
        mv /tpinstaller/work/watchdog.log.3 /tpinstaller/work/watchdog.log.4
        mv /tpinstaller/work/watchdog.log.2 /tpinstaller/work/watchdog.log.3
        mv /tpinstaller/work/watchdog.log /tpinstaller/work/watchdog.log.2
        # generate current watchdog report reason
        echo "Watchdog restart at `date`: ${REBOOT}" > /tpinstaller/work/watchdog.log
        if [ -f /tmp/ttwdog.reason ] ; then
            echo "=== Additional Crash Information ===" >> /tpinstaller/work/watchdog.log
            cat /tmp/ttwdog.reason >> /tpinstaller/work/watchdog.log
            echo "====================================" >> /tpinstaller/work/watchdog.log
        fi
        # stop the project
        /tpinstaller/tpctrl.sh stop
        sync
        reboot -d 5
        exit
    fi
}

echo "Watchdog daemon is started! Press Ctrl+C to stop..."
echo "(Note: First check is delayed by 10 minutes!)"
sleep 600
while true
do
    watchdog
    #echo "tich-tack at `date`"
    # check again after 5 min!
    sleep 300
done
