#!/bin/sh
#

TAR=rootfs
PACKAGES="all"
SRCDIR=/
DSTDIR=/tmp/install/rootfs
INSTALL=/tmp/install/install.sh
VERCFG=/tmp/install/tp-device.conf-default

rm -rf $DSTDIR
mkdir -p $DSTDIR

# generate /etc/tp-device.conf-default
echo "Generate version control information..."
/tpinstaller/tpctrl.sh genuid
if [ -f /etc/tp-device.conf-default ] ; then
    . /etc/tp-device.conf-default
fi
if [ -f /etc/tp-device.conf ] ; then
    . /etc/tp-device.conf
fi
echo "DEVICE_ARCH=\""$DEVICE_ARCH"\"" > $VERCFG
echo "DEVICE_VER=\""$DEVICE_VER"\"" >> $VERCFG
echo "DEVICE_VER_DATE=\"`date -u +%Y-%m-%d`\"" >> $VERCFG
echo "FIRMWARE_BUS=\""$FIRMWARE_BUS"\"" >> $VERCFG
echo "FIRMWARE_EIB=\""$FIRMWARE_EIB"\"" >> $VERCFG
echo "FIRMWARE_TP=\""$FIRMWARE_TP"\"" >> $VERCFG

# check dependencies
echo -n "Calculate dependencies "
while [ 1 ] ; do
    echo -n "."
    rm -f $DSTDIR/.dependencies
    for PKG in ${PACKAGES} ; do
        echo "$PKG" >> $DSTDIR/.dependencies
        if [ -f ${PKG}.dep ] ; then
            cat ${PKG}.dep >> $DSTDIR/.dependencies
            echo >> $DSTDIR/.dependencies
        fi
    done
    if [ -f $DSTDIR/.dependencies1 ] ; then
        cp -f $DSTDIR/.dependencies1 $DSTDIR/.dependencies2
        sort -b -u $DSTDIR/.dependencies > $DSTDIR/.dependencies1
        PACKAGES=$(/bin/cat $DSTDIR/.dependencies1)
        # check if both files are equal!
        diff -b -q $DSTDIR/.dependencies1 $DSTDIR/.dependencies2
        if [ $? -eq 0 ] ; then
            break
        fi
    else
        sort -b -u $DSTDIR/.dependencies > $DSTDIR/.dependencies1
        PACKAGES=$(/bin/cat $DSTDIR/.dependencies1)
    fi
done
rm -f $DSTDIR/.dependencies
rm -f $DSTDIR/.dependencies1
rm -f $DSTDIR/.dependencies2
echo " Done!"

cp -f install.beg $INSTALL
for PKG in ${PACKAGES} ; do
    if [ -f ${PKG}.beg ] ; then
        echo "# .beg install commands for package $PKG" >> $INSTALL
        echo "# *******************************" >> $INSTALL
        cat ${PKG}.beg >> $INSTALL
        echo " " >> $INSTALL
    fi
done
for PKG in ${PACKAGES} ; do
    echo -n "Process package: $PKG ... "
    
    echo "# *******************************" >> $INSTALL
    echo "# **" >> $INSTALL
    echo "# ** Package:$PKG" >> $INSTALL
    echo "# **" >> $INSTALL
    echo "# *******************************" >> $INSTALL
    
    if [ -f ${PKG}.pre ] ; then
        echo "# pre install commands for package $PKG" >> $INSTALL
        echo "# *******************************" >> $INSTALL
        cat ${PKG}.pre >> $INSTALL
        echo " " >> $INSTALL
    fi
    
    if [ -f ${PKG}.lst ] ; then
        echo "# copy files for package $PKG" >> $INSTALL
        echo "# *******************************" >> $INSTALL
            
        FILELIST=$(/bin/cat ${PKG}.lst)
    
        for F in ${FILELIST} ; do
            # remove quotes
            F=${F#\"}
            F=${F#\'}
            F=${F%\"}
            F=${F%\'}
            WILDCARD=0
            if echo "$F" | grep -q '[\*\?]' ; then
                WILDCARD=1
                DIR="/"
                for FF in $F ; do
                    DIR=$(dirname $FF)
                    break
                done
                mkdir -p $DSTDIR/$DIR
                cp -dpf $SRCDIR/$F $DSTDIR/$DIR
                echo "rm -f $F" >> $INSTALL
                echo "if [ ! -d $DIR ] ; then mkdir -p $DIR ; fi" >> $INSTALL
                echo "cp -dpf ./rootfs$F $DIR" >> $INSTALL
            else
                DIR=$(dirname $F)
                FIL=$(basename $F)
                mkdir -p $DSTDIR/$DIR
                cp -dpf $SRCDIR/$F $DSTDIR/$DIR
                echo "rm -f $F" >> $INSTALL
                echo "if [ ! -d $DIR ] ; then mkdir -p $DIR ; fi" >> $INSTALL
                echo "cp -dpf ./rootfs$F $DIR" >> $INSTALL
            fi
        done
        echo " " >> $INSTALL
    fi

    if [ -f ${PKG}.post ] ; then
        echo "# post install commands for package $PKG" >> $INSTALL
        echo "# *******************************" >> $INSTALL
        cat ${PKG}.post >> $INSTALL
        echo " " >> $INSTALL
    fi
    
    echo "Done!"
done
for PKG in ${PACKAGES} ; do
    if [ -f ${PKG}.end ] ; then
        echo "# .end install commands for package $PKG" >> $INSTALL
        echo "# *******************************" >> $INSTALL
        cat ${PKG}.end >> $INSTALL
        echo " " >> $INSTALL
    fi
done
cat install.end >> $INSTALL
chmod a+x $INSTALL
tar -zcf /tmp/firmware-upgrade.tar.gz -C /tmp install

